"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TreeIterator = exports.TreeNode = void 0;
var TreeNode = /** @class */ (function () {
    function TreeNode(key, value) {
        this.color = true;
        this.key = undefined;
        this.value = undefined;
        this.parent = undefined;
        this.brother = undefined;
        this.leftChild = undefined;
        this.rightChild = undefined;
        if (key !== undefined && value !== undefined) {
            this.key = key;
            this.value = value;
        }
    }
    TreeNode.prototype.rotateLeft = function () {
        var PP = this.parent;
        var PB = this.brother;
        var F = this.leftChild;
        var V = this.rightChild;
        if (!V)
            throw new Error("unknown error");
        var R = V.leftChild;
        var X = V.rightChild;
        if (PP && PP.key !== undefined) {
            if (PP.leftChild === this)
                PP.leftChild = V;
            else if (PP.rightChild === this)
                PP.rightChild = V;
        }
        V.parent = PP;
        V.brother = PB;
        V.leftChild = this;
        V.rightChild = X;
        if (PB)
            PB.brother = V;
        this.parent = V;
        this.brother = X;
        this.leftChild = F;
        this.rightChild = R;
        if (X) {
            X.parent = V;
            X.brother = this;
        }
        if (F) {
            F.parent = this;
            F.brother = R;
        }
        if (R) {
            R.parent = this;
            R.brother = F;
        }
        return V;
    };
    TreeNode.prototype.rotateRight = function () {
        var PP = this.parent;
        var PB = this.brother;
        var F = this.leftChild;
        if (!F)
            throw new Error("unknown error");
        var V = this.rightChild;
        var D = F.leftChild;
        var K = F.rightChild;
        if (PP && PP.key !== undefined) {
            if (PP.leftChild === this)
                PP.leftChild = F;
            else if (PP.rightChild === this)
                PP.rightChild = F;
        }
        F.parent = PP;
        F.brother = PB;
        F.leftChild = D;
        F.rightChild = this;
        if (PB)
            PB.brother = F;
        if (D) {
            D.parent = F;
            D.brother = this;
        }
        this.parent = F;
        this.brother = D;
        this.leftChild = K;
        this.rightChild = V;
        if (K) {
            K.parent = this;
            K.brother = V;
        }
        if (V) {
            V.parent = this;
            V.brother = K;
        }
        return F;
    };
    TreeNode.prototype.remove = function () {
        if (this.leftChild || this.rightChild)
            throw new Error("can only remove leaf node");
        if (this.parent) {
            if (this === this.parent.leftChild)
                this.parent.leftChild = undefined;
            else if (this === this.parent.rightChild)
                this.parent.rightChild = undefined;
        }
        if (this.brother)
            this.brother.brother = undefined;
        this.key = undefined;
        this.value = undefined;
        this.parent = undefined;
        this.brother = undefined;
    };
    TreeNode.TreeNodeColorType = {
        red: true,
        black: false
    };
    return TreeNode;
}());
exports.TreeNode = TreeNode;
function _TreeIterator(_node, header, iteratorType) {
    if (iteratorType === void 0) { iteratorType = 'normal'; }
    Object.defineProperties(this, {
        iteratorType: {
            value: iteratorType
        },
        node: {
            value: _node
        },
        pointer: {
            get: function () {
                if (_node.key === undefined) {
                    throw new Error("Tree iterator access denied!");
                }
                if (_node.value === undefined) {
                    return _node.key;
                }
                var obj = {};
                Object.defineProperties(obj, {
                    key: {
                        get: function () {
                            return _node.key;
                        },
                        enumerable: true,
                    },
                    value: {
                        get: function () {
                            return _node.value;
                        },
                        set: function (newValue) {
                            _node.value = newValue;
                        },
                        enumerable: true,
                    }
                });
                return obj;
            },
            enumerable: true
        }
    });
    this.equals = function (obj) {
        if (obj.constructor.name !== this.constructor.name) {
            throw new Error("obj's constructor is not ".concat(this.constructor.name, "!"));
        }
        if (this.iteratorType !== obj.iteratorType) {
            throw new Error("iterator type error!");
        }
        // @ts-ignore
        return _node === obj.node;
    };
    var _pre = function () {
        var _a;
        var preNode = _node;
        if (preNode.color === TreeNode.TreeNodeColorType.red && ((_a = preNode.parent) === null || _a === void 0 ? void 0 : _a.parent) === preNode) {
            preNode = preNode.rightChild;
        }
        else if (preNode.leftChild) {
            preNode = preNode.leftChild;
            while (preNode.rightChild)
                preNode = preNode === null || preNode === void 0 ? void 0 : preNode.rightChild;
        }
        else {
            var pre = (preNode === null || preNode === void 0 ? void 0 : preNode.parent) || undefined;
            while ((pre === null || pre === void 0 ? void 0 : pre.leftChild) === preNode) {
                preNode = pre;
                pre = (preNode === null || preNode === void 0 ? void 0 : preNode.parent) || undefined;
            }
            preNode = pre;
        }
        if (preNode === undefined)
            throw new Error("unknown error");
        return preNode;
    };
    var _next = function () {
        var nextNode = _node;
        if (nextNode === null || nextNode === void 0 ? void 0 : nextNode.rightChild) {
            nextNode = nextNode.rightChild;
            while (nextNode.leftChild)
                nextNode = nextNode === null || nextNode === void 0 ? void 0 : nextNode.leftChild;
        }
        else {
            var pre = (nextNode === null || nextNode === void 0 ? void 0 : nextNode.parent) || undefined;
            while ((pre === null || pre === void 0 ? void 0 : pre.rightChild) === nextNode) {
                nextNode = pre;
                pre = (nextNode === null || nextNode === void 0 ? void 0 : nextNode.parent) || undefined;
            }
            if (nextNode.rightChild !== pre) {
                nextNode = pre;
            }
        }
        if (nextNode === undefined)
            throw new Error("unknown error");
        return nextNode;
    };
    this.pre = function () {
        if (this.iteratorType === 'reverse') {
            if (_node === header.rightChild)
                throw new Error("Tree iterator access denied!");
            var preNode_1 = _next();
            return new exports.TreeIterator(preNode_1, header, this.iteratorType);
        }
        if (_node === header.leftChild)
            throw new Error("Tree iterator access denied!");
        var preNode = _pre();
        return new exports.TreeIterator(preNode, header);
    };
    this.next = function () {
        if (this.iteratorType === 'reverse') {
            if (_node === header)
                throw new Error("Tree iterator access denied!");
            var nextNode_1 = _pre();
            return new exports.TreeIterator(nextNode_1, header, this.iteratorType);
        }
        if (_node === header)
            throw new Error("Tree iterator access denied!");
        var nextNode = _next();
        return new exports.TreeIterator(nextNode, header);
    };
}
exports.TreeIterator = _TreeIterator;
