import { ContainerIterator, BaseType, Pair } from "../Base/Base";
export interface MapType<K, V> extends BaseType {
    /**
     * @return Iterator pointing to the begin element.
     */
    begin: () => ContainerIterator<Pair<K, V>>;
    /**
     * @return Iterator pointing to the super end like c++.
     */
    end: () => ContainerIterator<Pair<K, V>>;
    /**
     * @return Iterator pointing to the end element.
     */
    rBegin: () => ContainerIterator<Pair<K, V>>;
    /**
     * @return Iterator pointing to the super begin like c++.
     */
    rEnd: () => ContainerIterator<Pair<K, V>>;
    /**
     * @return The first element.
     */
    front: () => Pair<K, V> | undefined;
    /**
     * @return The last element.
     */
    back: () => Pair<K, V> | undefined;
    forEach: (callback: (element: Pair<K, V>, index: number) => void) => void;
    /**
     * @param element The element you want to find.
     * @return Iterator pointing to the element if found, or super end if not found.
     */
    find: (key: K) => ContainerIterator<Pair<K, V>>;
    /**
     * @return An iterator to the first element not less than the given key.
     */
    lowerBound: (key: K) => ContainerIterator<Pair<K, V>>;
    /**
     * @return An iterator to the first element greater than the given key.
     */
    upperBound: (key: K) => ContainerIterator<Pair<K, V>>;
    /**
     * @return An iterator to the first element not greater than the given key.
     */
    reverseLowerBound: (key: K) => ContainerIterator<Pair<K, V>>;
    /**
     * @return An iterator to the first element less than the given key.
     */
    reverseUpperBound: (key: K) => ContainerIterator<Pair<K, V>>;
    /**
     * Gets the key and value of the element at the specified position.
     */
    getElementByPos: (pos: number) => Pair<K, V>;
    /**
     * Gets the value of the element of the specified key.
     */
    getElementByKey: (key: K) => V | undefined;
    /**
     * Insert a new key-value pair or set value by key.
     */
    setElement: (key: K, value: V) => void;
    /**
     * Removes the elements at the specified position.
     */
    eraseElementByPos: (pos: number) => void;
    /**
     * Removes the elements of the specified key.
     */
    eraseElementByKey: (key: K) => void;
    /**
     * @return An iterator point to the next iterator.
     * Removes element by iterator.
     */
    eraseElementByIterator: (iter: ContainerIterator<Pair<K, V>>) => ContainerIterator<Pair<K, V>>;
    /**
     * Union the other Set to self.
     */
    union: (other: MapType<K, V>) => void;
    /**
     * @return The height of the RB-tree.
     */
    getHeight: () => number;
    /**
     * Using for 'for...of' syntax like Array.
     */
    [Symbol.iterator]: () => Generator<Pair<K, V>, void, undefined>;
}
declare const _default: new <K, V>(container?: {
    forEach: (callback: (element: Pair<K, V>) => void) => void;
}, cmp?: (x: K, y: K) => number) => MapType<K, V>;
export default _default;
